"use strict";
// Copyright IBM Corp. 2020. All Rights Reserved.
// Node module: @loopback/example-multi-tenancy
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiTenancyMiddlewareProvider = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const debug_1 = (0, tslib_1.__importDefault)(require("debug"));
const keys_1 = require("../keys");
const debug = (0, debug_1.default)('loopback:multi-tenancy');
/**
 * Provides the multi-tenancy action for a sequence
 */
let MultiTenancyMiddlewareProvider = class MultiTenancyMiddlewareProvider {
    constructor(getMultiTenancyStrategies, options = {
        strategyNames: ['header'],
    }) {
        this.getMultiTenancyStrategies = getMultiTenancyStrategies;
        this.options = options;
    }
    value() {
        return async (ctx, next) => {
            await this.action(ctx);
            return next();
        };
    }
    /**
     * The implementation of authenticate() sequence action.
     * @param request - The incoming request provided by the REST layer
     */
    async action(requestCtx) {
        debug('Identifying tenant for request %s', requestCtx.basePath);
        const tenancy = await this.identifyTenancy(requestCtx);
        if (tenancy == null)
            return;
        debug('Tenant identified by strategy %s', tenancy.strategy.name, tenancy.tenant);
        debug('Binding resources for tenant', tenancy.tenant);
        requestCtx.bind(keys_1.MultiTenancyBindings.CURRENT_TENANT).to(tenancy.tenant);
        await tenancy.strategy.bindResources(requestCtx, tenancy.tenant);
        return tenancy.tenant;
    }
    async identifyTenancy(requestCtx) {
        debug('Tenancy action is configured with', this.options);
        const strategyNames = this.options.strategyNames;
        let strategies = await this.getMultiTenancyStrategies();
        strategies = strategies
            .filter(s => strategyNames.includes(s.name))
            .sort((a, b) => {
            return strategyNames.indexOf(a.name) - strategyNames.indexOf(b.name);
        });
        if (debug.enabled) {
            debug('Tenancy strategies', strategies.map(s => s.name));
        }
        for (const strategy of strategies) {
            debug('Trying tenancy strategy %s', strategy.name);
            const tenant = await strategy.identifyTenant(requestCtx);
            if (tenant != null) {
                debug('Tenant is now identified by strategy %s', strategy.name, tenant);
                return { tenant, strategy };
            }
        }
        debug('No tenant is identified');
        return undefined;
    }
};
MultiTenancyMiddlewareProvider = (0, tslib_1.__decorate)([
    (0, core_1.extensionPoint)(keys_1.MULTI_TENANCY_STRATEGIES, {
        tags: {
            [core_1.ContextTags.KEY]: keys_1.MultiTenancyBindings.MIDDLEWARE,
        },
    }, (0, rest_1.asMiddleware)({
        group: 'tenancy',
        downstreamGroups: 'findRoute',
    })),
    (0, tslib_1.__param)(0, (0, core_1.extensions)()),
    (0, tslib_1.__param)(1, (0, core_1.config)()),
    (0, tslib_1.__metadata)("design:paramtypes", [Function, Object])
], MultiTenancyMiddlewareProvider);
exports.MultiTenancyMiddlewareProvider = MultiTenancyMiddlewareProvider;
//# sourceMappingURL=multi-tenancy-middleware.provider.js.map